
local arg1 = arg[1]	-- FASTA assuming Covid19 as 1 , SARS as 2

print(arg1)

local arg2 = arg[2]	-- Covid19
local offset1 = math.tointeger(arg[3])

print(arg2,offset1)

local arg3 = arg[4] -- SARS
local offset2 = math.tointeger(arg[5])

print(arg3,offset2)

local resfasta1 = math.tointeger(arg[6]) -- restrict to fasta sequence from this number
local resfasta2 = math.tointeger(arg[7]) -- to this one

print(resfasta1,resfasta2)

-- load fasta

local fasta = {}
do
	local file = io.open(arg1,'r')
	for l in file:lines() do
		if l:find("^>") then
			fasta[#fasta+1] = {}
			local f = fasta[#fasta]
			l = file:read()
			while l ~= '' do
				for c in l:gmatch('.') do f[#f+1] = c end
				l = file:read()
			end
		end
	end
	file:close()
end

print('Number of FASTA read:',#fasta)
for k=1,#fasta do print('Size of FASTA',k,':',#fasta[k]) end

-- OFFSET

-- Covid19

-- chain A : 598  - 1718
-- chain B : 1719 - 2839
-- chain C : 2840 - 3960 

-- SARS2002

-- chain A : 598  - 1699
-- chain B : 1700 - 2801
-- chain C : 2802 - 3903 

--local s1 = 1718 -- covid19  OFFSET RBD UP -- chain B
--local s2 = 597  -- SARS2002 OFFSET RBD UP -- chain A

--local offset1 = 597   -- covid19  OFFSET RBD DOWN -- chain A
--local offset2 = 2801  -- SARS2002 OFFSET RBD DOWN -- chain C

print("Offset1:",offset1)
print("Offset2:",offset2)

-- consensus residues

local consensus1 = {}
local consensus2 = {}
local fasta1 = fasta[1]
local fasta2 = fasta[2]
assert(#fasta1==#fasta2)
local map1,map2={},{}
print("FASTA")
do
	local s1,s2 = offset1,offset2
	for k=1,#fasta1 do
		if fasta1[k] ~= "." then s1=s1+1 map1[s1]=k end
		if fasta2[k] ~= "." then s2=s2+1 map2[s2]=k end
		if fasta1[k] == fasta2[k] then -- consensus
			consensus1[s1] = k
			consensus2[s2] = k
		end
		print(k, fasta1[k],s1, fasta2[k],s2, fasta1[k] == fasta2[k] and "<- - - consensus" or '')
	end
end

-- load residues contact frequency

local parse_frequency = function(fname)
	local buffer = {}
	print('Parsing',fname)
	local data = {}
	local f = assert(io.open(fname,'r'))
	for l in f:lines() do
		local t = {}
		for k in string.gmatch(l,'%S+') do t[#t+1] = k end -- split
		if #t == 6 then -- valid line with data from frequency file
			local res1 = string.sub(t[1],1,3) -- resname 1
			local res2 = string.sub(t[2],1,3) -- resname 2
			local id1 = math.tointeger(t[3]) -- resid 1
			local id2 = math.tointeger(t[4]) -- resid 2
			local freq = tonumber(t[5]) -- frequency
			data[#data+1] = {res1,res2,id1,id2,freq}
		end
	end
	f:close()
	return data
end

local data1 = parse_frequency(arg2)
local data2 = parse_frequency(arg3)

-- count number of contacts, separated by type

local category_contacts = function(data,consensus,map)
	local cc,cn,nc,nn = 0,0,0,0
	local fcc,fcn,fnc,fnn = 0.0,0.0,0.0,0.0
	for k=1,#data do
		local id1 = data[k][3]
		local id2 = data[k][4]
		local freq = data[k][5]
		if map[id1] and map[id2]
		   and map[id1] >= resfasta1 and map[id1] <= resfasta2
		   and map[id2] >= resfasta1 and map[id2] <= resfasta2
		then
			if         consensus[id1] and     consensus[id2] then cc=cc+1 fcc=fcc+freq
			elseif     consensus[id1] and not consensus[id2] then cn=cn+1 fcn=fcn+freq
			elseif not consensus[id1] and     consensus[id2] then nc=nc+1 fnc=fnc+freq
			else nn=nn+1 fnn=fnn+freq end
		end
	end
	--assert(cc+cn+nc+nn == #data)
	return cc,cn,nc,nn,fcc,fcn,fnc,fnn,cc+cn+nc+nn,fcc+fnc+fcn+fnn
end

print('Category, number of contacts 1, frequency sum 1:',category_contacts(data1,consensus1,map1))
print('Category, number of contacts 2, frequency sum 2:',category_contacts(data2,consensus2,map2))

-- count contacts

do
	local list1 = {}
	for k=1,#data1 do
		local id1  = data1[k][3]
		local id2  = data1[k][4]
		local freq = data1[k][5]
		local id1,id2 = math.min(id1,id2),math.max(id1,id2)
		if map1[id1] and map1[id2]
		   and map1[id1] >= resfasta1 and map1[id1] <= resfasta2
		   and map1[id2] >= resfasta1 and map1[id2] <= resfasta2
		   --and consensus1[id1] and consensus1[id2]
		   and freq > 0.5
		then
			local str = string.format("%d_%d",map1[id1],map1[id2])
			--assert(not list[str]) -- it's a new contact
			list1[str] = {'COVID19',table.unpack(data1[k])}
		end
	end
	local list2 = {}
	for k=1,#data2 do
		local id1  = data2[k][3]
		local id2  = data2[k][4]
		local freq = data2[k][5]
		local id1,id2 = math.min(id1,id2),math.max(id1,id2)
		if map2[id1] and map2[id2]
		   and map2[id1] >= resfasta1 and map2[id1] <= resfasta2
		   and map2[id2] >= resfasta1 and map2[id2] <= resfasta2
		   --and consensus2[id1] and consensus2[id2]
		  and freq > 0.5
		then
			local str = string.format("%d_%d",map2[id1],map2[id2])
			--if list[str] then list[str] = nil -- remove same contact
			--else list2[str] = {'SARS2002',table.unpack(data2[k])} end
			list2[str] = {'SARS2002',table.unpack(data2[k])}
		end
	end
	for k,v in pairs(list1) do
		if not list2[k] then
			local id1 = v[4]
			local id2 = v[5]
			local freq = v[6]
			local color = ''
			if freq > 0.5 then color = 'black' end
			if freq > 0.7 then color = 'black' end
			if freq > 0.9 then color = 'black' end
			print(k,v[1],"|",string.format("#0:%d@CA #0:%d@CA %s",id1,id2,color))
		end
	end
end

-- process data!

local process_data = function(data,consensus,map)
	local highfreq = {}
	local allfreq = {}
	local resname = {}
	for k=1,#data do
		local name = data[k][1]
		local seq  = data[k][3]
		local freq = data[k][5]
		--if map[seq] and map[seq] >= resfasta1 and map[seq] <= resfasta2 then
		if consensus[seq] then
			if freq > 0.9 then
				highfreq[seq] = (highfreq[seq] or 0.0) + freq
			end
			allfreq[seq] = (allfreq[seq] or 0.0) + freq
			resname[seq] = name
		end
		--end
	end
	return resname,allfreq,highfreq
end

local resname1,allfreq1,highfreq1 = process_data(data1,consensus1,map1)
local resname2,allfreq2,highfreq2 = process_data(data2,consensus2,map2)

-- print consensus contacts

do
	local s1,s2 = offset1,offset2
	for k=1,#fasta1 do
		if fasta1[k] ~= "." then s1 = s1+1 end
		if fasta2[k] ~= "." then s2 = s2+1 end
		if fasta1[k] == fasta2[k] then -- consensus
		--if resname1[s1] and resname2[s2] then -- restriction to map
			assert(k == consensus1[s1])
			assert(k == consensus2[s2])
			local fsum1 = allfreq1[s1] or 0.0
			local fsum2 = allfreq2[s2] or 0.0
			local fdiff = fsum1-fsum2
			local fdiffhigh = (highfreq1[s1] or 0.0) - (highfreq2[s2] or 0.0)
			print(string.format("C %6d | %1s %3s %6d | %1s %3s %6d | %10.4f %10.4f %10.4f %10.4f",k, fasta1[k],resname1[s1],s1, fasta2[k],resname2[s2],s2, fsum1,fsum2, fdiff,fdiffhigh ))
		--end
		end
	end
end

